/*Can4Open
linking functions between can4linux low level drivers and
the CanFestival CanOpen static libraries.
Nathan Z. Gustavson
ngustavson@emacinc.com

emac.inc
10/8/02
*/


#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "Can.h"
#include "can4linux.h"
#include "Can4Open.h"
#include "CanOpener.h"


logfunc txloggingfunc = NULL;
logfunc rxloggingfunc = NULL;
extern int busfd;
int wait = 0;/* blocking or non blocking IO*/
/*to use this it will have to be extern'd and modified
as it is in the example project Can4Opener. 
*/

/*msgtoMessage
conversion of the canmsg_t message structure(can4linux)
to the Message structure of CanOpenMatic(CanOpen for linux)
*/
int msgtoMessage(Message *message,canmsg_t *msg_t)
{
memcpy(&message->data,&msg_t->data,sizeof(message->data));
message->cob_id.w = (unsigned short)msg_t->id;
message->len = (char)msg_t->length;
	
if(msg_t->flags&MSG_RTR)
  message->rtr=1;
else
  message->rtr=0;

return 0;
}

/*Messagetomsg
conversion the Message structure of CanOpenMatic   
to the canmsg_t message structure(can4linux)
*/
int Messagetomsg(canmsg_t *msg_t, Message *message)
{
if(message->rtr)
msg_t->flags = MSG_RTR;
else
msg_t->flags = 0;

msg_t->length = (int)message->len;

memcpy(&msg_t->data,&message->data,sizeof(msg_t->data));
msg_t->id = (unsigned long)message->cob_id.w;

return 0;
}


/*
f_can_receive
calls read instead of ioctl to 
get data from can4linux
and translates the data into a message structure

returns 0 on success
 */
int f_can_receive(int notused, Message *m){
	int res;
	canmsg_t rx;
	
	if(wait)
	  while(!(res=read(busfd,&rx,1)));
	else
	  res = read(busfd,&rx,1);
    
	msgtoMessage(m,&rx);
			
	if(rxloggingfunc!=NULL)
	  {
	  if(res)
	    rxloggingfunc(m,STATE_Rx);
	  }
	return !res;
}


/*
f_can_send
translates a message stucture into a canmsg_t structure.
Calls write instead of ioctl to 
send data to can4linux

returns 0 on success
*/
int f_can_send(int notused, Message *m){
	int res;
	canmsg_t tx;
       	Messagetomsg(&tx,m);
	
	if(wait)
	  while(!(res=write(busfd,&tx,1))); 
	else
	  res = write(busfd,&tx,1);


	if(txloggingfunc!=NULL)
	   txloggingfunc(m, (res ? STATE_Tx : STATE_FAULT));
    	
       	return !res;
}


int f_can_open(const char *device){
	return open(device,O_RDWR);
}

int f_can_close(int fd){
	return close(fd);
}




